﻿using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using System.Xml.Linq;

namespace RemovingWebParts
{
    public static class FeatureHelper
    {

        public static void RemoveFeatureWebParts(SPFeatureReceiverProperties properties)
        {
            SPSite site = properties.Feature.Parent as SPSite;
            if (site == null)
            {
                SPWeb web = properties.Feature.Parent as SPWeb;
                if (web == null)
                    return;

                site = web.Site;
            }

            string wpListName = SPUtility.GetLocalizedString("$Resources:webpartgalleryList",
                                    "core", site.RootWeb.Language);
            SPList wpList = site.RootWeb.Lists.TryGetList(wpListName);
            if (wpList == null)
                return;

            List<string> webpartsInFeature = GetWebPartsInFeature(properties);
            foreach (string wp in webpartsInFeature)
            {
                string fileRelativeUrl = SPUrlUtility.CombineUrl(wpList.RootFolder.Url, wp);
                SPFile file = site.RootWeb.GetFile(fileRelativeUrl);
                if (file.Exists)
                    file.Delete();
            }
        }

        private static List<string> GetWebPartsInFeature(SPFeatureReceiverProperties properties)
        {
            XmlNode featureXml = properties.Feature.Definition.GetXmlDefinition(CultureInfo.CurrentCulture);
            XDocument xDoc = XDocument.Load(new XmlNodeReader(featureXml));
            XNamespace ns = "http://schemas.microsoft.com/sharepoint/";
            List<string> webparts = (from e in xDoc.Descendants(ns + "ElementFile")
                                     where e.Attribute("Location").Value.EndsWith(".webpart")
                                     select e.Attribute("Location").Value).ToList();

            List<string> webpartsFromUrl = new List<string>();
            foreach (string wp in webparts)
            {
                webpartsFromUrl.Add(GetWebPartFromUrl(properties, wp));
            }
            return webpartsFromUrl;
        }

        private static string GetWebPartFromUrl(SPFeatureReceiverProperties properties, string wp)
        {
            string[] pathSplit = wp.Split('\\');
            string path = Path.Combine(properties.Definition.RootDirectory, pathSplit[0]);
            using (FileStream file = File.OpenRead(Path.Combine(path, "elements.xml")))
            {
                XNamespace ns = "http://schemas.microsoft.com/sharepoint/";
                XDocument xDoc = XDocument.Load(file);
                return (from e in xDoc.Descendants(ns + "File")
                        where string.Compare(e.Attribute("Path").Value, wp, true) == 0
                        select e.Attribute("Url").Value).FirstOrDefault();
            }
        }

    }
}
